from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional
from datetime import datetime


@dataclass
class Message:
    role: str
    content: str
    timestamp: Optional[datetime] = None
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class Conversation:

    
    id: str
    title: str
    messages: List[Message] = field(default_factory=list)
    created_at: datetime = field(default_factory=datetime.now)
    updated_at: datetime = field(default_factory=datetime.now)
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class ModelConfig:

    
    name: str
    model_id: str
    temperature: float = 0.7
    max_tokens: int = 2048
    timeout: int = 30
    retry_attempts: int = 10
    additional_params: Dict[str, Any] = field(default_factory=dict)


@dataclass
class APIResponse:

    
    success: bool
    data: Optional[Any] = None
    error: Optional[str] = None
    status_code: int = 200
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class User:

    
    id: str
    username: str
    email: str
    created_at: datetime = field(default_factory=datetime.now)
    last_login: Optional[datetime] = None
    preferences: Dict[str, Any] = field(default_factory=dict)
    is_active: bool = True


@dataclass
class Token:

    
    access_token: str
    token_type: str = "Bearer"
    expires_in: Optional[int] = None
    refresh_token: Optional[str] = None
    scope: Optional[str] = None
    issued_at: datetime = field(default_factory=datetime.now)


@dataclass
class CompletionRequest:

    
    prompt: str
    model: str
    max_tokens: Optional[int] = None
    temperature: Optional[float] = None
    top_p: Optional[float] = None
    stop_sequences: List[str] = field(default_factory=list)
    user_id: Optional[str] = None
    conversation_id: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class CompletionResponse:

    
    id: str
    text: str
    model: str
    tokens_used: int
    finish_reason: str
    created_at: datetime = field(default_factory=datetime.now)
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class Error:

    
    code: str
    message: str
    details: Optional[str] = None
    timestamp: datetime = field(default_factory=datetime.now)
    trace_id: Optional[str] = None



class TitanGPTException(Exception):
    pass

class ConfigurationError(TitanGPTException):
    pass

class AuthenticationError(TitanGPTException):
    pass

class AuthorizationError(TitanGPTException):
    pass

class APIError(TitanGPTException):
    pass

class RateLimitError(APIError):
    pass

class ValidationError(TitanGPTException):
    pass

class ModelNotFoundError(TitanGPTException):
    pass

class PromptError(TitanGPTException):
    pass

class TimeoutError(TitanGPTException):
    pass

class ConnectionError(TitanGPTException):
    pass

class DataError(TitanGPTException):
    pass

class NotImplementedError(TitanGPTException):
    pass


import sys
from titangpt.client import TitanGPT
from titangpt.async_client import AsyncTitanGPT
from titangpt.exceptions import (
    TitanGPTException,
    APIError,
    AuthenticationError,
    AuthorizationError,
    ValidationError,
    RateLimitError,
    ModelNotFoundError,
    TimeoutError,
    ConnectionError
)

__version__ = "0.1.9"
__author__ = "TitanGPT"
__license__ = "MIT"

__all__ = [
    "TitanGPT",
    "AsyncTitanGPT",
    "TitanGPTException",
    "APIError",
    "AuthenticationError",
    "AuthorizationError",
    "ValidationError",
    "RateLimitError",
    "ModelNotFoundError",
    "TimeoutError",
    "ConnectionError"
]

_client = None

def get_client(api_key: str = None, base_url: str = "https://api.titangpt.ru") -> TitanGPT:
    global _client
    
    if _client is None:
        _client = TitanGPT(api_key=api_key, base_url=base_url)
    
    return _client

def set_api_key(api_key: str) -> None:
    global _client
    _client = TitanGPT(api_key=api_key)

_original_excepthook = sys.excepthook

def _titan_exception_hook(exc_type, exc_value, exc_traceback):
    if issubclass(exc_type, TitanGPTException):
        print(f"{exc_type.__name__}: {exc_value}")
    else:
        _original_excepthook(exc_type, exc_value, exc_traceback)

sys.excepthook = _titan_exception_hook


import os
import json
from typing import Any, Dict, Optional, List, Union
import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
from titangpt.exceptions import (
    APIError,
    AuthenticationError,
    RateLimitError,
    ValidationError,
    ModelNotFoundError,
    TitanGPTException
)

class TitanResponse(dict):
    def __getattr__(self, name):
        try:
            value = self[name]
            if isinstance(value, dict):
                return TitanResponse(value)
            if isinstance(value, list):
                return [TitanResponse(i) if isinstance(i, dict) else i for i in value]
            return value
        except KeyError:
            raise AttributeError(f"'TitanResponse' object has no attribute '{name}'")

class Completions:
    def __init__(self, client):
        self._client = client

    def create(self, model: str, messages: List[Dict[str, str]], **kwargs) -> TitanResponse:
        payload = {
            "model": model,
            "messages": messages,
            **kwargs
        }
        return self._client._post("v1/chat/completions", json=payload)

class Chat:
    def __init__(self, client):
        self.completions = Completions(client)

class Images:
    def __init__(self, client):
        self._client = client

    def generate(self, prompt: str, model: str = "flux", n: int = 1, size: str = "1024x1024", **kwargs) -> TitanResponse:
        payload = {
            "prompt": prompt,
            "model": model,
            "n": n,
            "size": size,
            **kwargs
        }
        return self._client._post("v1/images/generations", json=payload)

class Audio:
    def __init__(self, client):
        self.transcriptions = Transcriptions(client)

class Transcriptions:
    def __init__(self, client):
        self._client = client

    def create(self, file, model: str = "whisper-1", **kwargs) -> TitanResponse:
        if isinstance(file, str):
             with open(file, "rb") as f:
                 files = {"file": f}
                 data = {"model": model, **kwargs}
                 return self._client._post("v1/audio/transcriptions", files=files, data=data)
        
        files = {"file": file}
        data = {"model": model, **kwargs}
        return self._client._post("v1/audio/transcriptions", files=files, data=data)

class Music:
    def __init__(self, client):
        self._client = client
    
    def search(self, query: str) -> TitanResponse:
        return self._client._post("v2/music/search", json={"query": query})

    def lyrics(self, video_id: str) -> TitanResponse:
        return self._client._get(f"v2/music/lyrics/{video_id}")

    def download(self, video_id: str, save_path: str) -> str:
        response = self._client._get_binary(f"v2/music/download/{video_id}")
        
        if os.path.isdir(save_path):
            filename = f"{video_id}.mp3"
            save_path = os.path.join(save_path, filename)

        with open(save_path, "wb") as f:
            for chunk in response.iter_content(chunk_size=8192):
                f.write(chunk)
        return save_path

class Models:
    def __init__(self, client):
        self._client = client

    def list(self) -> TitanResponse:
        return self._client._post("v1/models")

class TitanGPT:
    def __init__(
        self,
        api_key: Optional[str] = None,
        base_url: str = "https://api.titangpt.ru",
        timeout: int = 60,
        max_retries: int = 3,
        user_id: Optional[str] = None
    ):
        self.api_key = api_key or os.getenv("TITANGPT_API_KEY")
        if not self.api_key:
            raise ValueError("The api_key client option must be set either by passing api_key to the client or by setting the TITANGPT_API_KEY environment variable")

        self.base_url = base_url.rstrip("/")
        self.timeout = timeout
        
        self.session = requests.Session()
        
        retry_strategy = Retry(
            total=max_retries,
            backoff_factor=1,
            status_forcelist=[500, 502, 503, 504],
            allowed_methods=["GET", "POST", "PUT", "DELETE"],
        )
        adapter = HTTPAdapter(max_retries=retry_strategy)
        self.session.mount("http://", adapter)
        self.session.mount("https://", adapter)
        
        auth_val = f"Bearer {self.api_key}"
        headers = {
            "Authorization": auth_val.encode('utf-8'), 
            "User-Agent": "TitanGPT-Python/1.0",
        }
        if user_id:
            headers["x-user-id"] = str(user_id)
            
        self.session.headers.update(headers)

        self.chat = Chat(self)
        self.images = Images(self)
        self.audio = Audio(self)
        self.music = Music(self)
        self.models = Models(self)

    def check_health(self) -> Dict[str, str]:
        url = f"{self.base_url}/" 
        try:
            response = self.session.get(url, timeout=10)
            if response.status_code == 200:
                return response.json()
            else:
                raise APIError(f"Health check failed with status {response.status_code}: {response.text}")
        except requests.exceptions.RequestException as e:
            raise APIError(f"Health check failed: {str(e)}")

    def _request(self, method: str, path: str, **kwargs) -> requests.Response:
        url = f"{self.base_url}/{path}"
        try:
            response = self.session.request(method, url, timeout=self.timeout, **kwargs)
            if response.status_code >= 400:
                self._handle_error(response)
            return response
        except requests.exceptions.RequestException as e:
            if isinstance(e, TitanGPTException):
                raise e
            raise APIError(f"Connection error: {str(e)}") from e
        except Exception as e:
            if isinstance(e, TitanGPTException):
                raise e
            raise APIError(f"Unexpected error: {str(e)}")

    def _post(self, path: str, json: dict = None, files=None, data=None) -> TitanResponse:
        response = self._request("POST", path, json=json, files=files, data=data)
        return TitanResponse(response.json())

    def _get(self, path: str, params: dict = None) -> TitanResponse:
        response = self._request("GET", path, params=params)
        return TitanResponse(response.json())

    def _get_binary(self, path: str) -> requests.Response:
        url = f"{self.base_url}/{path}"
        try:
            response = self.session.get(url, stream=True, timeout=self.timeout * 3)
            if response.status_code >= 400:
                self._handle_error(response)
            return response
        except requests.exceptions.RequestException as e:
            if isinstance(e, TitanGPTException):
                raise e
            raise APIError(f"Connection error: {str(e)}") from e

    def _handle_error(self, response):
        try:
            error_json = response.json()
            message = error_json.get("error", {}).get("message") or error_json.get("message")
            if not message and "detail" in error_json:
                message = error_json["detail"]
        except ValueError:
            message = response.text

        if not message:
            message = f"Error code: {response.status_code}"

        if response.status_code == 400:
            raise ValidationError(message)
        elif response.status_code == 401:
            raise AuthenticationError(f"Authentication failed: {message}")
        elif response.status_code == 403:
            raise AuthenticationError(f"Permission denied (Invalid API Key or Model): {message}")
        elif response.status_code == 404:
            raise ModelNotFoundError(message)
        elif response.status_code == 429:
            raise RateLimitError(message)
        else:
            raise APIError(f"TitanGPT API Error {response.status_code}: {message}")

    def close(self):
        self.session.close()

    def __enter__(self):
        return self
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()


from setuptools import setup, find_packages
from pathlib import Path

long_description = Path("README.md").read_text(encoding="utf-8")

setup(
    name="titangpt",
    version="0.1.9",
    author="TitanGPT",
    author_email="info@titangpt.ru",
    description="Official Python client for TitanGPT API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TitanGPT/titangpt",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",  
    ],
    python_requires=">=3.7",
    install_requires=[
        "requests>=2.28.0",
        "aiohttp>=3.8.0",
    ],
    include_package_data=True,
)



import os
import asyncio
import aiofiles
from typing import Any, Dict, Optional, List, Union
import httpx  
from titangpt.exceptions import (
    APIError,
    AuthenticationError,
    RateLimitError,
    ValidationError,
    ModelNotFoundError,
    TitanGPTException
)

class TitanResponse(dict):
    def __getattr__(self, name):
        try:
            value = self[name]
            if isinstance(value, dict):
                return TitanResponse(value)
            if isinstance(value, list):
                return [TitanResponse(i) if isinstance(i, dict) else i for i in value]
            return value
        except KeyError:
            raise AttributeError(f"'TitanResponse' object has no attribute '{name}'")

class AsyncCompletions:
    def __init__(self, client):
        self._client = client

    async def create(self, model: str, messages: List[Dict[str, str]], **kwargs) -> TitanResponse:
        payload = {
            "model": model,
            "messages": messages,
            **kwargs
        }
        return await self._client._post("v1/chat/completions", json=payload)

class AsyncChat:
    def __init__(self, client):
        self.completions = AsyncCompletions(client)

class AsyncImages:
    def __init__(self, client):
        self._client = client

    async def generate(self, prompt: str, model: str = "flux", n: int = 1, size: str = "1024x1024", **kwargs) -> TitanResponse:
        payload = {
            "prompt": prompt,
            "model": model,
            "n": n,
            "size": size,
            **kwargs
        }
        return await self._client._post("v1/images/generations", json=payload)

class AsyncAudio:
    def __init__(self, client):
        self.transcriptions = AsyncTranscriptions(client)

class AsyncTranscriptions:
    def __init__(self, client):
        self._client = client

    async def create(self, file, model: str = "whisper-1", **kwargs) -> TitanResponse:
        files = {}
        data = {'model': model}
        
        for k, v in kwargs.items():
            data[k] = str(v)

        file_obj = None
        should_close = False

        try:
            if isinstance(file, str):
                file_obj = open(file, 'rb')
                should_close = True
                files['file'] = file_obj
            else:
                files['file'] = file
            
            await self._client._ensure_client()
            return await self._client._request("POST", "v1/audio/transcriptions", data=data, files=files)
        finally:
            if should_close and file_obj:
                file_obj.close()

class AsyncMusic:
    def __init__(self, client):
        self._client = client
    
    async def search(self, query: str) -> TitanResponse:
        return await self._client._post("v2/music/search", json={"query": query})

    async def lyrics(self, video_id: str) -> TitanResponse:
        return await self._client._get(f"v2/music/lyrics/{video_id}")

    async def download(self, video_id: str, save_path: str) -> str:
        await self._client._ensure_client()
        url = f"{self._client.base_url}/v2/music/download/{video_id}"
        
        try:
            async with self._client._session.stream("GET", url, timeout=300.0) as resp:
                if resp.status_code >= 400:
                    await self._client._handle_error(resp)
                
                if os.path.isdir(save_path):
                    filename = f"{video_id}.mp3"
                    save_path = os.path.join(save_path, filename)
                
                async with aiofiles.open(save_path, mode='wb') as f:
                    async for chunk in resp.aiter_bytes(chunk_size=8192):
                        await f.write(chunk)
                
                return save_path
        except Exception as e:
            raise APIError(f"Download failed: {str(e)}")

class AsyncModels:
    def __init__(self, client):
        self._client = client

    async def list(self) -> TitanResponse:
        return await self._client._post("v1/models")

class AsyncTitanGPT:
    def __init__(
        self,
        api_key: Optional[str] = None,
        base_url: str = "https://api.titangpt.ru",
        timeout: int = 60,
        user_id: Optional[str] = None
    ):
        self.api_key = api_key or os.getenv("TITANGPT_API_KEY")
        if not self.api_key:
            raise ValueError("The api_key client option must be set either by passing api_key to the client or by setting the TITANGPT_API_KEY environment variable")

        self.base_url = base_url.rstrip("/")
        self.timeout = timeout
        self.user_id = user_id
        self._session: Optional[httpx.AsyncClient] = None 

        self.chat = AsyncChat(self)
        self.images = AsyncImages(self)
        self.audio = AsyncAudio(self)
        self.music = AsyncMusic(self)
        self.models = AsyncModels(self)

    async def _ensure_client(self):
        if self._session is None or self._session.is_closed:
            headers = {
                "Authorization": f"Bearer {self.api_key}",
                "User-Agent": "TitanGPT-Python-Async/1.0 (HTTP/2)"
            }
            if self.user_id:
                headers["x-user-id"] = str(self.user_id)
            
            self._session = httpx.AsyncClient(
                headers=headers, 
                http2=True,
                timeout=self.timeout
            )

    async def check_health(self) -> Dict[str, str]:
        await self._ensure_client()
        url = f"{self.base_url}/"
        try:
            response = await self._session.get(url, timeout=10.0)
            if response.status_code == 200:
                return response.json()
            text = response.text
            raise APIError(f"Health check failed with status {response.status_code}: {text}")
        except Exception as e:
            raise APIError(f"Health check failed: {str(e)}")

    async def _request(self, method: str, path: str, json: dict = None, data = None, params: dict = None, files = None) -> TitanResponse:
        await self._ensure_client()
        url = f"{self.base_url}/{path}"
        
        try:
            resp = await self._session.request(
                method, 
                url, 
                json=json, 
                data=data, 
                params=params, 
                files=files
            )
            
            if resp.status_code >= 400:
                await self._handle_error(resp)

            result = resp.json()
            return TitanResponse(result)

        except httpx.RequestError as e:
            raise APIError(f"Connection error: {str(e)}")
        except Exception as e:
            if isinstance(e, TitanGPTException):
                raise e
            raise APIError(f"Unexpected error: {str(e)}")

    async def _post(self, path: str, json: dict = None, data = None) -> TitanResponse:
        return await self._request("POST", path, json=json, data=data)

    async def _get(self, path: str, params: dict = None) -> TitanResponse:
        return await self._request("GET", path, params=params)

    async def _handle_error(self, response: httpx.Response):
        try:
            error_data = response.json()
            message = error_data.get("error", {}).get("message") or error_data.get("message")
            if not message and "detail" in error_data:
                message = error_data["detail"]
        except:
            message = response.text

        if not message:
            message = f"Error code: {response.status_code}"

        status = response.status_code

        if status == 400:
            raise ValidationError(message)
        elif status == 401:
            raise AuthenticationError(f"Authentication failed: {message}")
        elif status == 403:
            raise AuthenticationError(f"Permission denied (Invalid API Key): {message}")
        elif status == 404:
            raise ModelNotFoundError(message)
        elif status == 429:
            raise RateLimitError(message)
        else:
            raise APIError(f"TitanGPT API Error {status}: {message}")

    async def close(self):
        if self._session and not self._session.is_closed:
            await self._session.aclose()

    async def __aenter__(self):
        await self._ensure_client()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()